class Elephant {
    constructor(private readonly _name: string) {
    }

    toString() {
        return JSON.stringify(this);
    }
}

class Set_illustration<E> {
    private readonly _implementation: Set<E> = new Set;

    constructor(s?: Set<E>) {
        if (s === undefined)
            return;
        s.forEach((element: E) => {
            this._implementation.add(element);
        });
    }

    add(e: E) {
        return this._implementation.add(e);
    }

    erase(e: E) {
        return this._implementation.add(e);
    }

    size() {
        return this._implementation.size;
    }

    strict_subset(s: Set_illustration<E>) { // The "current" set is strictly included in 's'...
        for (const element of this._implementation)
            if (!s._implementation.has(element))
                return false;
        return s !== this && s._implementation.size !== this._implementation.size;
    }

    symmetrical_difference(s: Set_illustration<E>) { // All stuff that is neither in the one nor the other...
        const symmetrical_difference: Set_illustration<E> = new Set_illustration(this._implementation);
        s._implementation.forEach((element: E) => {
            symmetrical_difference.add(element); // Union
        });
        const intersection: Set_illustration<E> = new Set_illustration;
        s._implementation.forEach((element: E) => {
            if (this._implementation.has(element))
                intersection.add(element);
        });
        // Formally, symmetrical difference is "union - intersection":
        intersection._implementation.forEach((element: E) => {
            symmetrical_difference._implementation.delete(element);
        });
        return symmetrical_difference;
    }
}

(function () {
    let Babar = null, Dumbo = null, Jumbo = null;
    const zoo: Set_illustration<Elephant> = new Set_illustration;
    zoo.add(Babar); // Compilation error with '"strict": true'...
    zoo.add(Dumbo); // Compilation error with '"strict": true'...
    zoo.add(Jumbo); // Compilation error with '"strict": true'...
    console.log("'zoo.size' ** 1: " + zoo.size);
    Babar = new Elephant("Babar");
    console.log("'zoo.size' ** 2: " + zoo.size);
    zoo.add(Babar);
    zoo.add(Babar);
    const Babar_bis = new Elephant("Babar");
    zoo.add(Babar_bis);
    console.log("'zoo.size' ** 3: " + zoo.size);
    Dumbo = new Elephant("Dumbo");
    Jumbo = new Elephant("Jumbo");
    const zoo_annex: Set_illustration<Elephant> = new Set_illustration;
    zoo_annex.add(Babar);
    zoo_annex.add(Dumbo);
    zoo_annex.add(Jumbo);
    const symmetrical_difference = zoo.symmetrical_difference(zoo_annex);
    console.log("'zoo.symmetrical_difference(zoo_annex)': ");
    console.log("\n'zoo.strict_subset(zoo_annex)' ** 1: " + zoo.strict_subset(zoo_annex));
    console.log("'zoo_annex.strict_subset(zoo)' ** 1: " + zoo_annex.strict_subset(zoo));
    zoo.erase(null); // Compilation error with '"strict": true'...
    zoo.erase(Babar_bis);
    console.log("\n'zoo.strict_subset(zoo_annex)' ** 2: " + zoo.strict_subset(zoo_annex));
    console.log("'zoo_annex.strict_subset(zoo)' ** 2: " + zoo_annex.strict_subset(zoo));
})();
